==> fetch.php?media=uls%3Auls_update-1.9.5.tgz: update/von_1.9.4/sql/gettwertvb.sql <==
set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS gettwertvb//
create function gettwertvb(wid integer, von datetime(6), bis datetime(6), mode varchar(12))
                  returns varchar(200)
deterministic
sql security invoker
begin
  declare r, l varchar(20);
  declare ld datetime;
  declare f integer;
  declare t enum('str','num','dat');

  select type from werteindizes, teststepdetails, einheiten
      where werteindizes.ID=wid and teststepdetailsID=teststepdetails.ID
        and einheitID=einheiten.ID
      into t;
  if t = 'num' then
    case mode
      when 'min' then
            select min(wert+0) from twerte
            where werteindizesID = wid and von <= datum and datum <= bis
            into r;
      when 'max' then
            select max(wert+0) from twerte
            where werteindizesID = wid and von <= datum and datum <= bis
            into r;
      when 'avg' then
            select avg(wert+0), max(length(0+replace(wert, '.',''))) from twerte
            where werteindizesID = wid and von <= datum and datum <= bis
            into r, f;
            return round(r, f + 2 - locate('.', r));
      when 'last/avg' then
            select wert+0, datum from twerte
            where werteindizesID = wid and von <= datum and datum <= bis
            order by datum desc limit 1 into l, ld;
            select avg(wert+0), max(length(0+replace(wert, '.',''))) from twerte
            where werteindizesID = wid and von <= datum and datum < ld
            into r, f;
            if r != 0 then
              return round(l/r, f + 2 - locate('.', l/r));
            else
              return NULL;
            end if;
      when 'last/avg-1' then
            select wert+0, datum from twerte
            where werteindizesID = wid and von <= datum and datum <= bis
            order by datum desc limit 1 into l, ld;
            select avg(wert+0), max(length(0+replace(wert, '.',''))) from twerte
            where werteindizesID = wid and von <= datum and datum < ld
            into r, f;
            if r != 0 then
              return round(l/r-1, f + 2 - locate('.', l/r-1));
            else
              return NULL;
            end if;
      when 'dev' then
            select stddev_samp(wert+0) from twerte
            where werteindizesID = wid and von <= datum and datum <= bis
            into r;
      when 'sum' then
            select sum(wert+0), max(locate('.', reverse(wert))) from twerte
            where werteindizesID = wid and von <= datum and datum <= bis
            into r, f;
            if f < 0 then
              set f = 0;
            end if;
            return round(r, f);
      when 'max-min' then
            select max(wert+0)-min(wert+0) from twerte
            where werteindizesID = wid and von <= datum and datum <= bis
            into r;
      when 'last-first' then
            return (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum desc limit 1)
                 - (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum limit 1);
      when 'last-first+' then
            set r = (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum desc limit 1)
                 - (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum limit 1);
            if r < 0 then
              return NULL;
            end if;
      when 'last-first-' then
            set r = (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum desc limit 1)
                 - (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum limit 1);
            if r > 0 then
              return NULL;
            end if;
      when 'differ' then
            return BINARY (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum desc limit 1)
                != BINARY (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum limit 1);
      when 'last' then
            select wert from twerte
            where werteindizesID = wid and von <= datum and datum <= bis
            order by datum desc limit 1
            into r;
      when 'first' then
            select wert from twerte
            where werteindizesID = wid and von <= datum and datum <= bis
            order by datum limit 1
            into r;
      when 'size' then
            select length(wert) from twerte
            where werteindizesID = wid and von <= datum and datum <= bis
            order by datum desc limit 1
            into r;
      when 'firstsize' then
            select length(wert) from twerte
            where werteindizesID = wid and von <= datum and datum <= bis
            order by datum limit 1
            into r;
      when 'count' then
            select count(*) from twerte
            where werteindizesID = wid and von <= datum and datum <= bis
            into r;
      when 'grad' then
        return gradienttvb(wid, von, bis);
      when 'accel' then
        return accelt(wid, von, bis, '00:00:00', '24:00:00', '',
                      '', '', '', '', '', '', '', '', '', '', '');
      else
        return NULL;
    end case;
    return r;
  else
    case mode
      when 'min' then
        return (select min(wert) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis);
      when 'max' then
        return (select max(wert) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis);
      when 'count' then
        return (select count(*) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis);
      when 'last' then
        return (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                order by datum desc, ID desc limit 1);
      when 'first' then
        return (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                order by datum, ID limit 1);
      when 'size' then
        return (select length(wert) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                order by datum desc, ID desc limit 1);
      when 'firstsize' then
        return (select length(wert) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                order by datum, ID limit 1);
      when 'lastsize' then
        return (select length(wert) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                order by datum desc, ID desc limit 1);
      when 'differ' then
        return BINARY (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                order by datum desc, ID desc limit 1)
            != BINARY (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                order by datum, ID limit 1);
      else
        return '';
    end case;
  end if;
end//

GRANT EXECUTE on FUNCTION gettwertvb to logginglesen@'localhost', loggingtest@'localhost', loggingadmin@'localhost'//

==> fetch.php?media=uls%3Auls_update-1.9.6.tgz: update/von_1.9.5/sql/gettwertvb.sql <==
-- gettwertvb.sql
--
-- function gettwertvb(wid, von, bis, mode) returns varchar
-- wid: werteindizesID
-- von: datetime from
-- bis: datetime to
-- mode: aggregation function
-- return: aggragated value (table twerte)

set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS gettwertvb//
create function gettwertvb(wid integer, von datetime(6), bis datetime(6), mode varchar(12))
                  returns varchar(200)
deterministic
sql security invoker
begin
  declare r, l varchar(20);
  declare ld datetime;
  declare f integer;
  declare t enum('str','num','dat');

  select type into t from werteindizes, teststepdetails, einheiten
      where werteindizes.ID = wid and teststepdetailsID = teststepdetails.ID
        and einheitID = einheiten.ID;
  if t = 'num' then
    case mode
      when 'min' then
            select min(wert+0) into r from twerte
              where werteindizesID = wid and von <= datum and datum <= bis;
      when 'max' then
            select max(wert+0) into r from twerte
              where werteindizesID = wid and von <= datum and datum <= bis;
      when 'avg' then
            select avg(wert+0), max(length(0+replace(wert, '.',''))) into r, f from twerte
              where werteindizesID = wid and von <= datum and datum <= bis;
            return round(r, f + 2 - locate('.', r));
      when 'last/avg' then
            select wert+0, datum into l, ld from twerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum desc limit 1;
            select avg(wert+0), max(length(0+replace(wert, '.',''))) into r, f from twerte
              where werteindizesID = wid and von <= datum and datum < ld;
            if r != 0 then
              return round(l/r, f + 2 - locate('.', l/r));
            else
              return NULL;
            end if;
      when 'last/avg-1' then
            select wert+0, datum into l, ld from twerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum desc limit 1;
            select avg(wert+0), max(length(0+replace(wert, '.',''))) into r, f from twerte
              where werteindizesID = wid and von <= datum and datum < ld;
            if r != 0 then
              return round(l/r-1, f + 2 - locate('.', l/r-1));
            else
              return NULL;
            end if;
      when 'dev' then
            select stddev_samp(wert+0) into r from twerte
              where werteindizesID = wid and von <= datum and datum <= bis;
      when 'sum' then
            select sum(wert+0), max(locate('.', reverse(wert))) into r, f from twerte
              where werteindizesID = wid and von <= datum and datum <= bis;
            if f < 0 then
              set f = 0;
            end if;
            return round(r, f);
      when 'max-min' then
            select max(wert+0)-min(wert+0) into r from twerte
              where werteindizesID = wid and von <= datum and datum <= bis;
      when 'last-first' then
            return (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum desc limit 1)
                 - (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum limit 1);
      when 'last-first+' then
            set r = (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum desc limit 1)
                 - (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum limit 1);
            if r < 0 then
              return NULL;
            end if;
      when 'last-first-' then
            set r = (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum desc limit 1)
                 - (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum limit 1);
            if r > 0 then
              return NULL;
            end if;
      when 'differ' then
            return BINARY (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum desc limit 1)
                != BINARY (select wert from twerte
                    where werteindizesID = wid and von <= datum and datum <= bis
                    order by datum limit 1);
      when 'last' then
            select wert into r from twerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum desc limit 1;
      when 'first' then
            select wert into r from twerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum limit 1;
      when 'size' then
            select length(wert) into r from twerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum desc limit 1;
      when 'firstsize' then
            select length(wert) into r from twerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum limit 1;
      when 'count' then
            select count(*) into r from twerte
              where werteindizesID = wid and von <= datum and datum <= bis;
      when 'grad' then
        return gradienttvb(wid, von, bis);
      when 'accel' then
        return accelt(wid, von, bis, '00:00:00', '24:00:00', '',
                      '', '', '', '', '', '', '', '', '', '', '');
      else
        return NULL;
    end case;
    return r;
  else
    case mode
      when 'min' then
        return (select min(wert) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis);
      when 'max' then
        return (select max(wert) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis);
      when 'count' then
        return (select count(*) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis);
      when 'last' then
        return (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                order by datum desc, ID desc limit 1);
      when 'first' then
        return (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                order by datum, ID limit 1);
      when 'size' then
        return (select length(wert) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                order by datum desc, ID desc limit 1);
      when 'firstsize' then
        return (select length(wert) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                order by datum, ID limit 1);
      when 'lastsize' then
        return (select length(wert) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                order by datum desc, ID desc limit 1);
      when 'differ' then
        return BINARY (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                order by datum desc, ID desc limit 1)
            != BINARY (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                order by datum, ID limit 1);
      else
        return '';
    end case;
  end if;
end//

GRANT EXECUTE on FUNCTION gettwertvb to logginglesen@'localhost', loggingtest@'localhost', loggingadmin@'localhost'//
